@@ -1,12 +1,29 @@
+# Copyright (C) 2008 Jimmy Do <jimmydo@users.sourceforge.net>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 from gettext import gettext as _
 from gettext import ngettext
 from datetime import datetime, timedelta
+import threading
 import gtk
 import gtk.glade as glade
 import gtk.gdk as gdk
-import gnome
 import gnomeapplet
 import bonobo.ui
+import gst
 from timerapplet import core
 from timerapplet import ui
 from timerapplet import utils
@@ -69,6 +86,18 @@ class TimerApplet(object):
         self._manage_presets_dialog = manage_presets_dialog
         self._applet = applet
         self._timer = timer
+
+        self._gst_playbin = gst.element_factory_make('playbin', 'player')
+        def bus_event(bus, message):
+            t = message.type
+            if t == gst.MESSAGE_EOS:
+                self._gst_playbin.set_state(gst.STATE_NULL)
+            elif t == gst.MESSAGE_ERROR:
+                self._gst_playbin.set_state(gst.STATE_NULL)
+                err, debug == message.parse_error()
+                print 'Error playing sound: %s' % err, debug
+            return True
+        self._gst_playbin.get_bus().add_watch(bus_event)
         
         self._status_button = ui.StatusButton()
         self._notifier = ui.Notifier('TimerApplet', gtk.STOCK_DIALOG_INFO, self._status_button)
@@ -318,8 +347,11 @@ class TimerApplet(object):
         name = self._timer.get_name()
         self._status_button.set_label(construct_time_str(self._timer.get_remaining_time(),
                                                          show_all=False))
+
         fraction_remaining = float(self._timer.get_remaining_time()) / self._timer.get_duration()
-        self._status_button.set_progress(1.0 - fraction_remaining)
+        progress = min(1.0, max(0.0, 1.0 - fraction_remaining))
+        self._status_button.set_progress(progress)
+
         if len(name) > 0:
             # HH:MM:SS (<timer name>)
             self._status_button.set_tooltip(_('%02d:%02d:%02d (%s)') % (hours, minutes, seconds, name))
@@ -330,12 +362,13 @@ class TimerApplet(object):
     def _on_timer_state_changed(self, timer, data=None):
         print 'State changed'
         new_state = timer.get_state()
+        print '  new state: %d' % new_state
         
         # These actions should be done once upon a state change.
         # That's why they're done here and not in self._update_status_button();
         # self._update_status_button() could be called multiple times
         # while in the same state.
-        if new_state == core.Timer.STATE_FINISHED:    
+        if new_state == core.Timer.STATE_FINISHED:
             name = self._timer.get_name()
             end_time = self._timer.get_end_time()
             time_text = get_display_text_from_datetime(end_time)
@@ -378,6 +411,7 @@ class TimerApplet(object):
                 gdk.beep()
                 print 'Done playing beep.'
         else:
+            self._stop_sound()
             self._notifier.end()
             self._stop_pulsing_button()
         
@@ -448,8 +482,21 @@ class TimerApplet(object):
             sound_path = self._gconf.get_string(TimerApplet._CUSTOM_SOUND_PATH_KEY)
             
         print 'Playing notification sound: "%s"' % str(sound_path)
-        gnome.sound_play(sound_path)
-        print 'Done playing sound.'
+        self._play_sound(sound_path)
+        print 'Started playing notification sound.'
+
+    def _play_sound(self, file_path):
+        if not file_path:
+            print 'Invalid path to sound file'
+            return
+        self._gst_playbin.set_state(gst.STATE_NULL)
+        sound_uri = 'file://' + file_path
+        print 'Using GStreamer to play: ' + sound_uri
+        self._gst_playbin.set_property('uri', sound_uri)
+        self._gst_playbin.set_state(gst.STATE_PLAYING)
+
+    def _stop_sound(self):
+        self._gst_playbin.set_state(gst.STATE_NULL)
     
     def _start_pulsing_button(self):
         self._status_button.start_pulsing()
